@@ -81,12 +81,16 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
             use checkout_core::dropshot;
             use checkout_core::dropshot::endpoint;
             use checkout_core::dropshot::RequestContext;
+            use checkout_core::dropshot::Path;
             use checkout_core::dropshot::TypedBody;
             use checkout_core::dropshot::HttpError;
             use checkout_core::dropshot::HttpResponseOk;
             use checkout_core::http::StatusCode;
+            use checkout_core::uuid::Uuid;
             use checkout_core::checkout::Checkout;
-            use checkout_core::server::CreateRequest;
+            use checkout_core::server::StandardPathParams;
+            use checkout_core::server::CreateBodyParams;
+            use checkout_core::server::AddOrRemoveItemBodyParams;
             use checkout_core::store::CheckoutStore;
             use checkout_core::transaction::TransactionController;
             use checkout_core::context::Context;
@@ -102,9 +106,9 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
             }]
             pub async fn create_handler(
                 req_ctx: Arc<RequestContext<ServerContext>>,
-                body_params: TypedBody<CreateRequest>,
+                body: TypedBody<CreateBodyParams>,
             ) -> Result<HttpResponseOk<Checkout>, HttpError> {
-                let params = body_params.into_inner();
+                let params = body.into_inner();
                 let mut ctx = req_ctx
                     .context()
                     .context_provider
@@ -130,6 +134,70 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                     }
                 }
             }
+
+            #[endpoint {
+                method = PATCH,
+                path = "/checkout/{id}/add_item",
+            }]
+            pub async fn add_item_handler(
+                req_ctx: Arc<RequestContext<ServerContext>>,
+                path: Path<StandardPathParams>,
+                body: TypedBody<AddOrRemoveItemBodyParams>,
+            ) -> Result<HttpResponseOk<Checkout>, HttpError> {
+                let path_params = path.into_inner();
+                let body_params = body.into_inner();
+                let mut ctx = req_ctx
+                    .context()
+                    .context_provider
+                    .new_context()
+                    .await
+                    .unwrap();
+
+                ctx.start_transaction().await.unwrap();
+
+                let result = ctx.get_checkout(&path_params.id).await.map_err(|err| Err(HttpError {
+                    status_code: StatusCode::BAD_REQUEST,
+                    error_code: Some(err.code.clone()),
+                    external_message: err.message.clone(),
+                    internal_message: err.message.clone(),
+                }));
+
+                if let Err(_) = result {
+                    ctx.abort_transaction().await.unwrap();
+                    return result;
+                }
+
+                let maybe_co = result.unwrap();
+
+                if maybe_co.is_none() {
+                    ctx.abort_transaction().await.unwrap();
+                    return Err(HttpError {
+                        status_code: StatusCode::NOT_FOUND,
+                        error_code: Some("404".to_string()),
+                        external_message: "the requested checkout could not be found".to_string(),
+                        internal_message: "the requested checkout could not be found".to_string(),
+                    });
+                }
+
+                let mut co = maybe_co.unwrap();
+
+                match co.add_item(&mut ctx, body_params.sku.clone(), body_params.quantity).await {
+                    Ok(_) => {
+                        ctx.set_checkout(&co.id, &co).await.unwrap();
+                        ctx.commit_transaction().await.unwrap();
+                        Ok(HttpResponseOk(co))
+                    }
+                    Err(err) => {
+                        ctx.abort_transaction().await.unwrap();
+                        Err(HttpError {
+                            status_code: StatusCode::BAD_REQUEST,
+                            error_code: Some(err.code.clone()),
+                            external_message: err.message.clone(),
+                            internal_message: err.message.clone(),
+                        })
+                    }
+                }
+            }
         }
 
         impl #impl_generics checkout_server::Server<#common_lifetime, #context_struct_name #ty_generics> for #server_struct_name {
@@ -138,6 +206,7 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
             fn api_description() -> checkout_server::ApiDescription<checkout_server::ServerContext> {
                 let mut api = checkout_server::ApiDescription::<checkout_server::ServerContext>::new();
                 api.register(checkout_server::create_handler).unwrap();
+                api.register(checkout_server::add_item_handler).unwrap();
                 api
             }
 
